CREATE TABLE npcDiscount (
  discountID INT IDENTITY NOT NULL,
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  description TEXT,
  isActive BIT NOT NULL DEFAULT '1',
  usersAppliesType VARCHAR(32) DEFAULT NULL,
  usersAppliesValue VARCHAR(255) DEFAULT NULL,
  itemsAppliesType VARCHAR(32) DEFAULT NULL,
  itemsAppliesValue VARCHAR(255) DEFAULT NULL,
  rewardType VARCHAR(32) DEFAULT NULL,
  reward DECIMAL(10,4) NULL,
  couponCode VARCHAR(100) DEFAULT NULL,
  startDate VARCHAR(100) DEFAULT NULL,
  expirationDate VARCHAR(100) DEFAULT NULL,
  timesUsed INT DEFAULT NULL,
  maxTimesUsed INT DEFAULT NULL,
  isTimesPerCustomer BIT NOT NULL DEFAULT '1',
  discountOrder INT NOT NULL DEFAULT '0',
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.DiscountBean',
  CONSTRAINT PK_discountID PRIMARY KEY CLUSTERED 
  ( discountID ) ON [PRIMARY],
  CONSTRAINT AK_discountCode UNIQUE NONCLUSTERED 
  ( code ) ON [PRIMARY]
);

CREATE INDEX IDX_npcDiscount_name ON npcDiscount (name);
CREATE INDEX IDX_npcDiscount_discountOrder ON npcDiscount (discountOrder);

CREATE TABLE npcSKUDiscount (
  skuDiscountID INT IDENTITY NOT NULL,
  skuID INT NOT NULL DEFAULT '0',
  discountID INT NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.SKUDiscountBean',
  CONSTRAINT PK_skuDiscountID PRIMARY KEY CLUSTERED 
  ( skuDiscountID ) ON [PRIMARY],
  CONSTRAINT AK_skuDiscount UNIQUE NONCLUSTERED 
  ( skuID, discountID ) ON [PRIMARY],
  CONSTRAINT FK_sku_skuDiscount FOREIGN KEY (
	skuID
  ) REFERENCES npcSKU (
	skuID
  ) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT FK_discount_skuDiscount FOREIGN KEY (
	discountID
  ) REFERENCES npcDiscount (
	discountID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);


CREATE TABLE npcDiscountRange (
  discountRangeID INT IDENTITY NOT NULL,
  discountID INT NOT NULL,
  floor DECIMAL(10,4) NOT NULL DEFAULT '0',
  ceiling DECIMAL(10,4) NOT NULL DEFAULT '0',
  rewardType VARCHAR(32) DEFAULT NULL,
  reward DECIMAL(10,4) NOT NULL DEFAULT '0',
  isActive BIT NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.DiscountRangeBean',
  CONSTRAINT discountRangeID PRIMARY KEY CLUSTERED 
  ( discountRangeID ) ON [PRIMARY],
  CONSTRAINT FK_discount_discountRange FOREIGN KEY (
	discountID
  ) REFERENCES npcDiscount (
	discountID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX IDX_npcDiscountRange_reward ON npcDiscountRange (reward);


CREATE TABLE npcOrderDiscount (  
  orderDiscountID INT IDENTITY NOT NULL,
  orderID INT NOT NULL DEFAULT '0',
  orderItemID INT NULL,
  total DECIMAL(10,4) DEFAULT NULL,
  discountID INT NOT NULL DEFAULT '0',
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  usersAppliesType VARCHAR(32) DEFAULT NULL,
  usersAppliesValue VARCHAR(255) DEFAULT NULL,
  itemsAppliesType VARCHAR(32) DEFAULT NULL,
  itemsAppliesValue VARCHAR(255) DEFAULT NULL,
  rewardType VARCHAR(32) DEFAULT NULL,
  reward DECIMAL(10,4) NOT NULL DEFAULT '0',
  couponCode VARCHAR(100) DEFAULT NULL,
  floor DECIMAL(10,4) NOT NULL DEFAULT '0',
  ceiling DECIMAL(10,4) NOT NULL DEFAULT '0',
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderDiscountBean',
  CONSTRAINT PK_orderDiscountID PRIMARY KEY CLUSTERED 
  ( orderDiscountID ) ON [PRIMARY],
  CONSTRAINT FK_order_orderDiscount FOREIGN KEY (
	orderID
  ) REFERENCES npcOrder (
	orderID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX IDX_npcOrderDiscount_total ON npcOrderDiscount (total);
CREATE INDEX IDX_npcOrderDiscount_code ON npcOrderDiscount (code);
CREATE INDEX IDX_npcOrderDiscount_name ON npcOrderDiscount (name);

CREATE TABLE npcRelatedProduct (
  relatedProductID INT IDENTITY NOT NULL,
  parentProductID INT NOT NULL DEFAULT '0',
  productID INT NOT NULL DEFAULT '0',
  relatedType VARCHAR(32) DEFAULT NULL,
  relatedProductOrder INT DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.RelatedProductBean',
  CONSTRAINT PK_relatedProductID PRIMARY KEY CLUSTERED 
  ( relatedProductID ) ON [PRIMARY],
  CONSTRAINT AK_relatedProduct UNIQUE NONCLUSTERED 
  ( parentProductID, productID ) ON [PRIMARY],
  CONSTRAINT FK_product_relatedProduct FOREIGN KEY (
	parentProductID
  ) REFERENCES npcProduct (
	productID
  ) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT FK_relatedProduct_product FOREIGN KEY (
	productID
  ) REFERENCES npcProduct (
	productID
  )
);
