CREATE TABLE sscShippingRule (
  shippingRuleID INT IDENTITY NOT NULL,
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  description TEXT,
  isActive BIT NOT NULL DEFAULT '1',
  ruleType VARCHAR(32) DEFAULT NULL,
  ruleOrder INT NOT NULL DEFAULT '0',
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.shipping.ShippingRuleBean',
  CONSTRAINT shippingRuleID PRIMARY KEY CLUSTERED 
  ( shippingRuleID ) ON [PRIMARY],
  CONSTRAINT AK_sscShippingRuleCode UNIQUE NONCLUSTERED 
  ( code ) ON [PRIMARY]
);

CREATE INDEX IDX_sscShippingRule_name ON sscShippingRule (name);
CREATE INDEX IDX_sscShippingRule_ruleOrder ON sscShippingRule (ruleOrder);

CREATE TABLE sscSKUShippingRule (
  skuShippingRuleID INT IDENTITY NOT NULL,
  skuID INT NOT NULL,
  shippingRuleID INT NOT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.shipping.SKUShippingRuleBean',
  CONSTRAINT skuShippingRuleID PRIMARY KEY CLUSTERED 
  ( skuShippingRuleID ) ON [PRIMARY],
  CONSTRAINT AK_skuShippingRule UNIQUE NONCLUSTERED 
  ( skuID, shippingRuleID ) ON [PRIMARY],
  CONSTRAINT FK_sku_skuShippingRule FOREIGN KEY (
	skuID
  ) REFERENCES sscSKU (
	skuID
  ) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT FK_shippingRule_skuShippingRule FOREIGN KEY (
	shippingRuleID
  ) REFERENCES sscShippingRule (
	shippingRuleID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE sscShippingRuleRange (
  shippingRuleRangeID INT IDENTITY NOT NULL,
  shippingRuleID INT NOT NULL,
  floor DECIMAL(10,4) NOT NULL DEFAULT '0',
  ceiling DECIMAL(10,4) NOT NULL DEFAULT '0',
  isActive BIT NOT NULL DEFAULT '1',
  discountType VARCHAR(32) DEFAULT NULL,
  discount DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgDescription TEXT,
  pkgWeight DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgLength DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgHeight DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgWidth DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgExtra1 VARCHAR(255) DEFAULT NULL,
  pkgExtra2 VARCHAR(255) DEFAULT NULL,
  pkgExtra3 VARCHAR(255) DEFAULT NULL,
  pkgExtra4 VARCHAR(255) DEFAULT NULL,
  pkgExtra5 VARCHAR(255) DEFAULT NULL,
  limitMethods TEXT,
  limitType VARCHAR(32) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.shipping.ShippingRuleRangeBean',
  CONSTRAINT shippingRuleRangeID PRIMARY KEY CLUSTERED 
  ( shippingRuleRangeID ) ON [PRIMARY],
  CONSTRAINT FK_shippingRuleRange_shippingRule FOREIGN KEY (
	shippingRuleID
  ) REFERENCES sscShippingRule (
	shippingRuleID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE sscOrderShippingRule (  
  orderShippingRuleID INT IDENTITY NOT NULL,
  orderID INT NOT NULL,
  orderItemID INT DEFAULT NULL,
  total DECIMAL(10,4) DEFAULT NULL,
  shippingRuleID INT NOT NULL DEFAULT '0',
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  ruleType VARCHAR(32) DEFAULT NULL,
  floor DECIMAL(10,4) DEFAULT NULL,
  ceiling DECIMAL(10,4) DEFAULT NULL,
  discountType VARCHAR(32) DEFAULT NULL,
  discount DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgDescription TEXT,
  pkgWeight DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgLength DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgHeight DECIMAL(10,4) NOT NULL DEFAULT '0',
  pkgWidth DECIMAL(10,4) NOT NULL DEFAULT '0',
  limitMethods TEXT,
  limitType VARCHAR(32) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  pkgExtra1 VARCHAR(255) DEFAULT NULL,
  pkgExtra2 VARCHAR(255) DEFAULT NULL,
  pkgExtra3 VARCHAR(255) DEFAULT NULL,
  pkgExtra4 VARCHAR(255) DEFAULT NULL,
  pkgExtra5 VARCHAR(255) DEFAULT NULL,
  subtotal DECIMAL(10,4) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderShippingRuleBean',
  CONSTRAINT orderShippingRuleID PRIMARY KEY CLUSTERED 
  ( orderShippingRuleID ) ON [PRIMARY],
  CONSTRAINT FK_order_orderShippingRule FOREIGN KEY (
	orderID
  ) REFERENCES sscOrder (
	orderID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX IDX_sscOrderShippingRule_orderItemID ON sscOrderShippingRule (orderItemID);
CREATE INDEX IDX_sscOrderShippingRule_total ON sscOrderShippingRule (total);
CREATE INDEX IDX_sscOrderShippingRule_code ON sscOrderShippingRule (code);
CREATE INDEX IDX_sscOrderShippingRule_name ON sscOrderShippingRule (name);
