package com.softslate.commerce.administrator.ordermanagement;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.User;
import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.businessobjects.order.CartDiscountProcessor;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderDiscount;
import com.softslate.commerce.businessobjects.order.OrderItem;
import com.softslate.commerce.businessobjects.order.OrderProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseForm;
import com.softslate.commerce.customer.order.CheckoutUtils;

/**
 * <code>OrderManagementUtilsImpl</code> is the default implementation of the
 * <code>OrderManagementUtils</code> interface for the application.
 * 
 * @author David Tobey
 * @see com.softslate.commerce.customer.core.ActionUtils
 * 
 */
public class OrderManagementUtilsImpl implements OrderManagementUtils {

	static Log log = LogFactory.getLog(OrderManagementUtilsImpl.class);

	public Map findPreviousShippingOption(Collection options, Order order,
			BaseForm baseForm) {

		String shippingMethod = order.getShippingMethod();
		String statusDetails = order.firstDelivery().getStatusDetails();
		boolean currentOptionSpecified = (shippingMethod != null && shippingMethod
				.length() > 0)
				|| (statusDetails != null && statusDetails.length() > 0);
		Map previousOption = null;
		if (options != null && currentOptionSpecified) {

			// First try to find it by its name
			String storeShippingMethodNameWithOrder = (String) baseForm
					.getSettings().getValue("storeShippingMethodNameWithOrder");
			if (storeShippingMethodNameWithOrder != null
					&& storeShippingMethodNameWithOrder.equals("1")) {
				Iterator i = options.iterator();
				while (i.hasNext()) {
					Map option = (Map) i.next();
					String name = (String) option.get("name");
					if (shippingMethod != null && shippingMethod.equals(name)) {
						previousOption = option;
						break;
					}
				}
			}

			// If that failed then try to find it by its code
			if (previousOption == null) {
				Iterator i = options.iterator();
				while (i.hasNext()) {
					Map option = (Map) i.next();
					String code = (String) option.get("code");
					if ((shippingMethod != null && shippingMethod.equals(code))
							|| ((shippingMethod == null || shippingMethod
									.length() == 0) && statusDetails != null && statusDetails
									.equals(code))) {
						previousOption = option;
						break;
					}
				}

			}
		}
		if (previousOption == null && options != null && !options.isEmpty()
				&& currentOptionSpecified) {
			String message = "The shipping method '"
					+ order.getShippingMethod()
					+ "' is no longer valid for the order. Please update "
					+ "the shipping method if appropriate.";
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("messages.detail", message));
			saveMessages(baseForm.getRequest(),
					(ActionMessages) baseForm.getMessages());
		} else if (previousOption == null
				&& (options == null || options.isEmpty())
				&& currentOptionSpecified) {
			String message = "There are now "
					+ "no valid shipping methods available for this order. "
					+ "Please adjust the shipping and order totals manually "
					+ "if appropriate.";
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.detail", message));
			saveMessages(baseForm.getRequest(),
					(ActionMessages) baseForm.getMessages());
		} else if (previousOption == null
				&& (options == null || options.isEmpty())
				&& !currentOptionSpecified) {
			String message = "There are "
					+ "no valid shipping methods available for this order. "
					+ "(Is the delivery address entered and complete?)";
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.detail", message));
			saveMessages(baseForm.getRequest(),
					(ActionMessages) baseForm.getMessages());
		}
		return previousOption;
	}

	public Collection findCouponCodes(Order order) {
		Collection couponCodes = new ArrayList();
		Collection orderDiscounts = order.getOrderDiscounts();
		if (orderDiscounts != null && !orderDiscounts.isEmpty()) {
			Iterator i = orderDiscounts.iterator();
			while (i.hasNext()) {
				OrderDiscount od = (OrderDiscount) i.next();
				if (od.getCouponCode() != null
						&& od.getCouponCode().length() > 0) {
					if (log.isDebugEnabled())
						log.debug("Adding existing coupon code from order: "
								+ od.getCouponCode());
					couponCodes.add(od.getCouponCode());
				}
			}
		}
		return couponCodes;
	}

	public Map findOrderChanges(Order order, Order originalOrder,
			Collection newOrderItems) {
		Map changes = new HashMap();
		if (order.getShippingMethod() != null
				&& !order.getShippingMethod().equals(
						originalOrder.getShippingMethod())) {
			String changeValue = "New";
			if (originalOrder.getShippingMethod() != null
					&& originalOrder.getShippingMethod().length() > 0) {
				changeValue = originalOrder.getShippingMethod();
			}
			changes.put("Order.shippingMethod", changeValue);
		}
		if (order.firstDelivery() != null
				&& originalOrder.firstDelivery() != null
				&& order.firstDelivery().getStatusDetails() != null
				&& !order
						.firstDelivery()
						.getStatusDetails()
						.equals(originalOrder.firstDelivery()
								.getStatusDetails())) {
			String changeValue = "New";
			if (originalOrder.firstDelivery().getStatusDetails() != null
					&& originalOrder.firstDelivery().getStatusDetails()
							.length() > 0) {
				changeValue = originalOrder.getShippingMethod();
			}
			changes.put("Order.firstDelivery.statusDetails", changeValue);
		}
		if (order.getSubtotal() != null
				&& !order.getSubtotal().equals(originalOrder.getSubtotal())) {
			changes.put("Order.subtotal", originalOrder.getSubtotal());
		}
		if (order.getShipping() != null
				&& !order.getShipping().equals(originalOrder.getShipping())) {
			changes.put("Order.shipping", originalOrder.getShipping());
		}
		if (order.getTax() != null
				&& !order.getTax().equals(originalOrder.getTax())) {
			changes.put("Order.tax", originalOrder.getTax());
		}
		if (order.getTotal() != null
				&& !order.getTotal().equals(originalOrder.getTotal())) {
			changes.put("Order.total", originalOrder.getTotal());
		}
		if (order.getOrderDiscounts() != null) {
			Collection origDiscountsMatchedUp = new ArrayList();
			Iterator i = order.getOrderDiscounts().iterator();
			while (i.hasNext()) {
				OrderDiscount od = (OrderDiscount) i.next();
				boolean thisDisountChanged = true;
				if (originalOrder.getOrderDiscounts() != null) {
					Iterator j = originalOrder.getOrderDiscounts().iterator();
					while (j.hasNext()) {
						OrderDiscount origOD = (OrderDiscount) j.next();
						if (od.getCode() != null
								&& od.getCode().equals(origOD.getCode())
								&& od.getTotal() != null
								&& od.getTotal().equals(origOD.getTotal())) {
							thisDisountChanged = false;
							origDiscountsMatchedUp.add(origOD);
							break;
						}
					}
				}
				if (thisDisountChanged) {
					changes.put("Order.orderDiscounts." + od.getDiscountID(),
							"Changed");
				}

			}
		}
		if (newOrderItems != null) {
			Iterator i = newOrderItems.iterator();
			while (i.hasNext()) {
				OrderItem oi = (OrderItem) i.next();
				changes.put("Order.orderItems." + oi.getOrderItemID(), "New");
			}
		}
		if (log.isDebugEnabled())
			log.debug("orderChanges: " + changes);
		return changes;
	}

	protected void saveErrors(HttpServletRequest request, ActionMessages errors) {

		// Remove any error messages attribute if none are required
		if ((errors == null) || errors.isEmpty()) {
			request.removeAttribute(Globals.ERROR_KEY);
			return;
		}

		// Save the error messages we need
		request.setAttribute(Globals.ERROR_KEY, errors);
	}

	protected void saveMessages(HttpServletRequest request,
			ActionMessages messages) {

		// Remove any messages attribute if none are required
		if ((messages == null) || messages.isEmpty()) {
			request.removeAttribute(Globals.MESSAGE_KEY);
			return;
		}

		// Save the messages we need
		request.setAttribute(Globals.MESSAGE_KEY, messages);
	}

	public String wrapAction(BaseOrderManagementAction action, BaseForm baseForm)
			throws Exception {

		// We'll use the common checkout methods defined in ActionUtils,
		// and OrderProcessor
		ActionUtils actionUtils = (ActionUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"actionUtilsImplementer");
		CheckoutUtils checkoutUtils = (CheckoutUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"checkoutUtilsImplementer");
		OrderProcessor orderProcessor = (OrderProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"orderProcessorImplementer");
		CustomerProcessor customerProcessor = (CustomerProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"customerProcessorImplementer");

		try {
			// Keep track of the new items in case we have to roll them back
			Collection newOrderItems = null;

			// Load the order being edited
			if (log.isInfoEnabled())
				log.info("Loading the order");
			Order order = orderProcessor.loadOrderFromID(PropertyUtils
					.describe(baseForm));
			baseForm.getRequest().setAttribute("order", order);
			if (order == null) {
				if (log.isInfoEnabled())
					log.info("Exiting because order not found: "
							+ PropertyUtils.getProperty(baseForm, "orderID"));
				baseForm.getErrors().add(
						ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.detail",
								"This order no longer exists. Order ID: "
										+ PropertyUtils.getProperty(baseForm,
												"orderID")));
				saveErrors(baseForm.getRequest(),
						(ActionMessages) baseForm.getErrors());
				return "failure";
			}

			// Set the session's order, customer, and enteredCouponCodes
			baseForm.getUser().setOrder(order);
			baseForm.getUser().setEnteredCouponCodes(findCouponCodes(order));

			// Load the customer associated with the order
			if (order.getCustomerID() != null
					&& order.getCustomerID().intValue() != 0) {
				Map customerID = new HashMap();
				customerID.put("customerID", order.getCustomerID());
				Customer customer = customerProcessor
						.loadCustomerFromID(customerID);
				baseForm.getUser().setCustomer(customer);
			} else {
				baseForm.getUser().setCustomer(null);
			}

			// Duplicate the order to a detached object so we can compare
			// what's changed
			Order dupeOrder = orderProcessor.duplicateOrder(order);
			baseForm.getRequest().setAttribute("dupeOrder", dupeOrder);

			// Put a user object and the baseForm object in the request
			// scope with the order under it for orderEmails.jsp
			User user =  baseForm.getBusinessObjectFactory()
					.createObject(User.class);
			user.setLocale(baseForm.getUser().getLocale());
			user.setCustomer(baseForm.getUser().getCustomer());
			user.setOrder(order);
			baseForm.getRequest().setAttribute("user", user);
			baseForm.getRequest().setAttribute("baseForm", baseForm);

			Map results = performWrappedAction(action, baseForm);

			if (results != null) {
				// Return immediately if a forward was sent back
				if (results.get("returnForward") != null) {
					return (String) results.get("returnForward");
				}

				// Assign newOrderItems in case there are any new ones -
				// used later for findOrderChanges
				newOrderItems = (Collection) results.get("newOrderItems");

				// Process inventory results (eg. sending the low stock
				// emails)
				if (log.isInfoEnabled())
					log.info("Processing inventory results");
				String forward = actionUtils.processCartAddResults(results,
						baseForm);

				// Check for errors (if the inventory check finds that no
				// items are in stock it will have removed the item)
				if (!forward.startsWith("success")) {
					if (log.isInfoEnabled())
						log.info("Exiting because of inventory result: "
								+ forward);
					return forward;
				}
			}

			// At this point, the item has been added/removed and discounts
			// and inventory have been processed.

			// Process just taxes or taxes plus shipping
			boolean processShipping = false;

			// Don't process shipping if it's not required
			String shippingRequired = (String) baseForm.getSettings().getValue(
					"shippingRequired");
			if (shippingRequired != null && shippingRequired.equals("1")) {

				// Load the valid shipping methods for the order as it now
				// stands
				if (log.isInfoEnabled())
					log.info("Loading valid shipping options");
				Collection options = checkoutUtils
						.loadShippingOptions(baseForm);

				// If a shipping option was not submitted, try to find the
				// shipping option that the customer had previously chosen.
				String shippingOption = (String) PropertyUtils.getProperty(
						baseForm, "shippingOption");
				if (shippingOption == null || shippingOption.length() == 0) {
					Map previousOption = findPreviousShippingOption(options,
							order, baseForm);
					if (previousOption != null) {
						shippingOption = previousOption.get("processorClass")
								+ "|" + previousOption.get("code");
						PropertyUtils.setProperty(baseForm, "shippingOption",
								shippingOption);
					}
				}

				// If no option was found, there's some sort of error so
				// skipping shipping procesing
				if (shippingOption != null && shippingOption.length() > 0) {
					processShipping = true;
				}

			}

			// Now reprocess shipping and/or taxes
			if (processShipping) {
				if (log.isInfoEnabled())
					log.info("Processing shipping and taxes");
				ActionForward af = checkoutUtils
						.processCheckoutShipping(baseForm);

				// Check for any errors - ActionUtils will already save
				// messages in the request, we just need to return if there
				// are any.
				if (!af.getName().startsWith("success")) {
					if (log.isInfoEnabled())
						log.info("Shipping processing error: " + af.getName());
				}
			} else {

				// Just reprocess the taxes
				if (log.isInfoEnabled())
					log.info("Just processing tax.");
				Map taxResults = checkoutUtils.processTax(baseForm);

				// Check for errors
				if (Integer.parseInt((String) taxResults.get("resultCode")) != 0) {
					if (log.isInfoEnabled())
						log.info("Failed to process tax: " + taxResults);
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.detail", "Failed to process tax. "
									+ taxResults.get("resultsMessage")));
					saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				}

				// Reprocess discounts so gift certificates may be
				// calculated against the order total
				CartDiscountProcessor dp = (CartDiscountProcessor) baseForm
						.getBusinessObjectFactory().createObject(
								"cartDiscountProcessorImplementer");
				Map discountResults = dp.processAfterTaxes(PropertyUtils
						.describe(baseForm));
				if (log.isDebugEnabled() && discountResults != null) {
					log.debug("Discount results: " + discountResults);
				}
			}

			// Put the order in the request scope for display
			order = orderProcessor.loadOrderFromID(PropertyUtils
					.describe(baseForm));
			baseForm.getRequest().setAttribute("order", order);

			// Create a Map of what changed within the order
			Map orderChanges = findOrderChanges(order, dupeOrder, newOrderItems);
			baseForm.getRequest().setAttribute("orderChanges", orderChanges);

			if (log.isInfoEnabled())
				log.info("Returning success.");

			return "success";

		} catch (Exception e) {
			if (log.isErrorEnabled())
				log.error("Exception follows: ", e);
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.detail", e.toString()));
			saveErrors(baseForm.getRequest(),
					(ActionMessages) baseForm.getErrors());
			return "exception";
		}
	}

	public Map performWrappedAction(BaseOrderManagementAction action,
			BaseForm baseForm) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing subclass processing.");
		return action.processOrderManagementAction(baseForm);
	}
}
