package com.softslate.commerce.administrator.product;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.SKU;
import com.softslate.commerce.businessobjects.product.SKUProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to requests to
 * "/administrator/SKUAdd.do". Processes a request to add a record to the
 * system.
 * 
 * @author David Tobey
 */
public class SKUAddAction extends BaseAction {

	static Log log = LogFactory.getLog(SKUAddAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		SKUAddEditForm baseForm = (SKUAddEditForm) form;

		// Handle generating the number from the name if it is empty
		ActionUtils actionUtils = (ActionUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"actionUtilsImplementer");
		if ((baseForm.getSkuNumber() == null || baseForm.getSkuNumber().equals(
				""))
				&& (baseForm.getSkuName() != null && !baseForm.getSkuName()
						.equals(""))) {
			baseForm.setSkuNumber(actionUtils.generateCodeFromName(baseForm
					.getSkuName()));
		}

		// Add the record
		SKUProcessor processor = (SKUProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"skuProcessorImplementer");
		Map results = processor.addSKU(PropertyUtils.describe(form));

		// Successful insert
		if (results == null || results.get("resultCode") == null
				|| results.get("resultCode").equals("0")) {

			// Set the success messages
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("messages.recordAdded"));
			if (results != null && results.get("newRecord") != null) {
				SKU newRecord = (SKU) results.get("newRecord");

				// entity.jsp wants the new record in the request scope.
				request.setAttribute("sku", newRecord);
				String params = "";
				if (request.getParameter("productID") != null) {
					params = params + "&productID="
							+ request.getParameter("productID");
				}
				if (request.getParameter("parentIDField") != null) {
					params = params + "&parentIDField="
							+ request.getParameter("parentIDField");
				}
				baseForm.getMessages().add(
						ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("messages.editNewRecord",
								"SKUEditForm.do?skuID=" + newRecord.getSkuID()
										+ params));
			}
			saveMessages(baseForm.getRequest(), baseForm.getMessages());

			// If successful, signal the form to be reinstantiated.
			request.setAttribute("reset", "true");

			return mapping.findForward("success");
		}

		// Set the error messages
		if (results != null && results.get("existingRecord") != null) {
			SKU existingRecord = (SKU) results.get("existingRecord");
			String duplicateField = (String) results.get("duplicateField");
			String existingCode = existingRecord.getSkuNumber();
			if (duplicateField.equals("productID")) {
				existingCode = existingRecord.getProductID().toString();
			}
			baseForm.getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.resultCode"
							+ results.get("resultCode"), duplicateField,
							existingCode));
			baseForm.getErrors()
					.add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.editExistingRecord",
									"SKUEditForm.do?skuID="
											+ existingRecord.getSkuID(),
									duplicateField, existingCode));
			saveErrors(baseForm.getRequest(),
					(ActionMessages) baseForm.getErrors());
		}
		return mapping.findForward("success");
	}
}
