package com.softslate.commerce.businessobjects.customer;

import java.util.Collection;
import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with customer delivery
 * addresses. Created in the Struts layer in response to requests for viewing,
 * editing, adding, and deleting delivery addresses.
 * <p>
 * When the application needs to create an instance that implements
 * <code>CustomerAddressProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "customerAddressProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "customerAddressProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.customer.CustomerAddressProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface CustomerAddressProcessor extends BusinessProcessor {

	abstract Map loadCustomerAddressesAndCount(Map parameters) throws Exception;

	abstract Map addCustomerAddress(Map parameters) throws Exception;

	abstract CustomerAddress loadCustomerAddressFromID(Map parameters)
			throws Exception;

	abstract Map editCustomerAddress(Map parameters) throws Exception;

	abstract Map editDeleteCustomerAddresses(Map parameters) throws Exception;

	abstract void deleteCustomerAddress(Map parameters) throws Exception;

	abstract Map addAddressBookAddress(Map parameters, boolean updateDatabase) throws Exception;

	abstract Map addAddressBookAddress(Map parameters) throws Exception;

	abstract Map deleteAddressBookAddress(Map parameters) throws Exception;

	abstract Map updateAddressBookAddress(Map parameters) throws Exception;

	abstract CustomerAddress findCustomerAddress(
			Collection<CustomerAddress> customerAddresses, int customerAddressID);

	/**
	 * Takes parameters (usually describe from an address form) and returns only
	 * those starting with billing or shipping.
	 * 
	 * @param parameters
	 * @param type
	 * @return
	 */
	Map describeDeliveryOrBillingForAddressBook(Map parameters, String type, boolean removeType);
}
