package com.softslate.commerce.businessobjects.order;

import java.util.Collection;
import java.util.Locale;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing an order delivery within the system. An order delivery
 * corresponds to a single destination of items within an order. The system
 * currently does not fully support the notion of multiple deliveries per order.
 * However, some aspects of this future feature are built into the application,
 * namely the database table and supporting classes.
 * <p>
 * When the application needs to create an instance that implements
 * <code>OrderDelivery</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "orderDeliveryImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "orderDeliveryImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.OrderDeliveryBean}.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudOrderDelivery")
public interface OrderDelivery extends BusinessObject {
	abstract int getOrderDeliveryID();

	abstract void setOrderDeliveryID(int orderDeliveryID);

	abstract int getOrderID();

	abstract void setOrderID(int orderID);

	abstract String getTrackingNumber();

	abstract void setTrackingNumber(String trackingNumber);

	abstract Double getTotal();

	abstract void setTotal(Double total);

	abstract Double getShipping();

	abstract void setShipping(Double shipping);

	abstract Double getTax();

	abstract void setTax(Double tax);

	abstract Double getTaxableSubtotal();

	abstract void setTaxableSubtotal(Double taxableSubtotal);

	abstract Double getSubtotal();

	abstract void setSubtotal(Double subtotal);

	abstract Double getWeight();

	abstract void setWeight(Double weight);

	abstract Integer getQuantity();

	abstract void setQuantity(Integer quantity);

	abstract String getFormattedTotal();

	abstract void setFormattedTotal(String formattedTotal);

	abstract String getFormattedShipping();

	abstract void setFormattedShipping(String formattedShipping);

	abstract String getFormattedTax();

	abstract void setFormattedTax(String formattedTax);

	abstract String getFormattedTaxableSubtotal();

	abstract void setFormattedTaxableSubtotal(String formattedTaxableSubtotal);

	abstract String getFormattedSubtotal();

	abstract void setFormattedSubtotal(String formattedSubtotal);

	abstract String getLastModified();

	abstract void setLastModified(String lastModified);

	abstract String getFormattedLastModified();

	abstract void setFormattedLastModified(String formattedLastModified);

	abstract String getDelivered();

	abstract void setDelivered(String delivered);

	abstract String getFormattedDelivered();

	abstract void setFormattedDelivered(String formattedDelivered);

	abstract String getStatus();

	abstract void setStatus(String status);

	abstract String getStatusDetails();

	abstract void setStatusDetails(String statusDetails);

	abstract String getFirstName();

	abstract void setFirstName(String firstName);

	abstract String getLastName();

	abstract void setLastName(String lastName);

	abstract String getOrganization();

	abstract void setOrganization(String organization);

	abstract String getAddress1();

	abstract void setAddress1(String address1);

	abstract String getAddress2();

	abstract void setAddress2(String address2);

	abstract String getCity();

	abstract void setCity(String city);

	abstract String getState();

	abstract void setState(String state);

	abstract String getCountry();

	abstract void setCountry(String country);

	abstract String getPostalCode();

	abstract void setPostalCode(String postalCode);

	abstract String getPhone1();

	abstract void setPhone1(String phone1);

	abstract String getPhone2();

	abstract void setPhone2(String phone2);

	abstract String getEmail1();

	abstract void setEmail1(String email1);

	abstract String getEmail2();

	abstract void setEmail2(String email2);

	abstract String getExtra1();

	abstract void setExtra1(String extra1);

	abstract String getExtra2();

	abstract void setExtra2(String extra2);

	abstract String getExtra3();

	abstract void setExtra3(String extra3);

	abstract String getExtra4();

	abstract void setExtra4(String extra4);

	abstract String getExtra5();

	abstract void setExtra5(String extra5);

	abstract String getNotes();

	abstract void setNotes(String notes);

	public Boolean getIsResidential();

	public void setIsResidential(Boolean isResidential);

	abstract Collection getOrderItemCollection();

	abstract void setOrderItemCollection(Collection orderItemCollection);

	abstract String getDeliveryFirstName();

	abstract void setDeliveryFirstName(String deliveryFirstName);

	abstract String getDeliveryLastName();

	abstract void setDeliveryLastName(String deliveryLastName);

	abstract String getDeliveryOrganization();

	abstract void setDeliveryOrganization(String deliveryOrganization);

	abstract String getDeliveryAddress1();

	abstract void setDeliveryAddress1(String deliveryAddress1);

	abstract String getDeliveryAddress2();

	abstract void setDeliveryAddress2(String deliveryAddress2);

	abstract String getDeliveryCity();

	abstract void setDeliveryCity(String deliveryCity);

	abstract String getDeliveryState();

	abstract void setDeliveryState(String deliveryState);

	abstract String getDeliveryCountry();

	abstract void setDeliveryCountry(String deliveryCountry);

	abstract String getDeliveryPostalCode();

	abstract void setDeliveryPostalCode(String deliveryPostalCode);

	abstract String getDeliveryPhone1();

	abstract void setDeliveryPhone1(String deliveryPhone1);

	abstract String getDeliveryPhone2();

	abstract void setDeliveryPhone2(String deliveryPhone2);

	abstract String getDeliveryEmail1();

	abstract void setDeliveryEmail1(String deliveryEmail1);

	abstract String getDeliveryEmail2();

	abstract void setDeliveryEmail2(String deliveryEmail2);

	abstract String getDeliveryExtra1();

	abstract void setDeliveryExtra1(String deliveryExtra1);

	abstract String getDeliveryExtra2();

	abstract void setDeliveryExtra2(String deliveryExtra2);

	abstract String getDeliveryExtra3();

	abstract void setDeliveryExtra3(String deliveryExtra3);

	abstract String getDeliveryExtra4();

	abstract void setDeliveryExtra4(String deliveryExtra4);

	abstract String getDeliveryExtra5();

	abstract void setDeliveryExtra5(String deliveryExtra5);

	abstract String getDeliveryNotes();

	abstract void setDeliveryNotes(String deliveryNotes);

	abstract Boolean getDeliveryIsResidential();

	abstract void setDeliveryIsResidential(Boolean isResidential);

	abstract Order getOrder();

	abstract void setOrder(Order order);

	abstract void addFormatting(Locale locale, String currencyCode);
}
