package com.softslate.commerce.businessobjects.order;

import java.io.Serializable;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;
import com.softslate.commerce.businessobjects.product.Attribute;
import com.softslate.commerce.businessobjects.product.Option;

/**
 * Object representing a order item attribute within the system.
 * <code>OrderItemAttributeBean</code> is the default implementation of the
 * <code>OrderItemAttribute</code> interface for the application.
 * <p>
 * The database stores order item attributes in the
 * <code>sscOrderItemAttribute</code> table, and this interface can be used to
 * represent a single row of that table. The Hibernate configuration file
 * <code>OrderItemAttribute.hbm.xml</code> maps that table's columns to the
 * properties in this object.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudOrderItemAttribute")
public class OrderItemAttributeBean extends BaseBusinessObject implements
		OrderItemAttribute, Serializable {

	private static final long serialVersionUID = -2047623075003850709L;

	private int orderItemAttributeID = 0;

	private int orderItemID = 0;

	private Double total = null;

	private Double weight = null;

	private Double costTotal = null;

	private String attributeValue = null;

	private int attributeID = 0;

	private String attributeCode = null;

	private String attributeName = null;

	private Double attributeUnitCost = null;

	private Double attributeUnitPrice = null;

	private Double attributeAltPrice = null;

	private Double attributeWeight = null;

	private String attributeImage = null;

	private int optionID = 0;

	private String optionCode = null;

	private String optionName = null;

	private Double optionUnitCost = null;

	private Double optionUnitPrice = null;

	private Double optionAltPrice = null;

	private Double optionWeight = null;

	private String optionImage = null;

	private OrderItem orderItem = null;

	private Attribute attribute = null;

	private Option option = null;

	public Double getCostTotal() {
		return costTotal;
	}

	public void setCostTotal(Double costTotal) {
		this.costTotal = costTotal;
	}

	public OrderItem getOrderItem() {
		return orderItem;
	}

	public void setOrderItem(OrderItem orderItem) {
		this.orderItem = orderItem;
	}

	public Attribute getAttribute() {
		return attribute;
	}

	public void setAttribute(Attribute attribute) {
		this.attribute = attribute;
	}

	public Option getOption() {
		return option;
	}

	public void setOption(Option option) {
		this.option = option;
	}

	public int getOrderItemAttributeID() {
		return orderItemAttributeID;
	}

	public void setOrderItemAttributeID(int orderItemAttributeID) {
		this.orderItemAttributeID = orderItemAttributeID;
	}

	public Double getAttributeAltPrice() {
		return attributeAltPrice;
	}

	public void setAttributeAltPrice(Double attributeAltPrice) {
		this.attributeAltPrice = attributeAltPrice;
	}

	public String getAttributeCode() {
		return attributeCode;
	}

	public void setAttributeCode(String attributeCode) {
		this.attributeCode = attributeCode;
	}

	public int getAttributeID() {
		return attributeID;
	}

	public void setAttributeID(int attributeID) {
		this.attributeID = attributeID;
	}

	public String getAttributeName() {
		return attributeName;
	}

	public void setAttributeName(String attributeName) {
		this.attributeName = attributeName;
	}

	public Double getAttributeUnitCost() {
		return attributeUnitCost;
	}

	public void setAttributeUnitCost(Double attributeUnitCost) {
		this.attributeUnitCost = attributeUnitCost;
	}

	public Double getAttributeUnitPrice() {
		return attributeUnitPrice;
	}

	public void setAttributeUnitPrice(Double attributeUnitPrice) {
		this.attributeUnitPrice = attributeUnitPrice;
	}

	public String getAttributeValue() {
		return attributeValue;
	}

	public void setAttributeValue(String attributeValue) {
		this.attributeValue = attributeValue;
	}

	public Double getAttributeWeight() {
		return attributeWeight;
	}

	public void setAttributeWeight(Double attributeWeight) {
		this.attributeWeight = attributeWeight;
	}

	public Double getOptionAltPrice() {
		return optionAltPrice;
	}

	public void setOptionAltPrice(Double optionAltPrice) {
		this.optionAltPrice = optionAltPrice;
	}

	public String getOptionCode() {
		return optionCode;
	}

	public void setOptionCode(String optionCode) {
		this.optionCode = optionCode;
	}

	public int getOptionID() {
		return optionID;
	}

	public void setOptionID(int optionID) {
		this.optionID = optionID;
	}

	public String getOptionName() {
		return optionName;
	}

	public void setOptionName(String optionName) {
		this.optionName = optionName;
	}

	public Double getOptionUnitCost() {
		return optionUnitCost;
	}

	public void setOptionUnitCost(Double optionUnitCost) {
		this.optionUnitCost = optionUnitCost;
	}

	public Double getOptionUnitPrice() {
		return optionUnitPrice;
	}

	public void setOptionUnitPrice(Double optionUnitPrice) {
		this.optionUnitPrice = optionUnitPrice;
	}

	public Double getOptionWeight() {
		return optionWeight;
	}

	public void setOptionWeight(Double optionWeight) {
		this.optionWeight = optionWeight;
	}

	public int getOrderItemID() {
		return orderItemID;
	}

	public void setOrderItemID(int orderItemID) {
		this.orderItemID = orderItemID;
	}

	public Double getTotal() {
		return total;
	}

	public void setTotal(Double total) {
		this.total = total;
	}

	public Double getWeight() {
		return weight;
	}

	public void setWeight(Double weight) {
		this.weight = weight;
	}

	public String getAttributeImage() {
		return attributeImage;
	}

	public void setAttributeImage(String attributeImage) {
		this.attributeImage = attributeImage;
	}

	public String getOptionImage() {
		return optionImage;
	}

	public void setOptionImage(String optionImage) {
		this.optionImage = optionImage;
	}
	
}
