package com.softslate.commerce.businessobjects.payment;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.payment.paypal.PayPalConstants;

/**
 * Processes a payment in the store for a given user's cart. This implementation
 * processes a request through PayPal Payflow Pro's ACH processing for
 * electronic payments. The {@link #process(Map)} method manages the process of
 * sending a request to PayPal, and handling the response appropriately.
 * 
 * @author Ashish Jain
 * @see com.softslate.commerce.daos.payment.PaymentDAO
 */
public class PayflowProACHProcessor extends BasePayPalNVPProcessor implements
		PaymentProcessor {

	static Log log = LogFactory.getLog(PayflowProACHProcessor.class);

	public Map process(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing payment.");

		parameters.put(PayPalConstants.TENDER, PayPalConstants.ACH_TENDER_TYPE);
		parameters.put(PayPalConstants.TRXTYPE, PayPalConstants.ACH_TRX_TYPE);

		String response = this.makePayPalCall(parameters, "payflowPro");

		if (log.isInfoEnabled())
			log.info("response: " + response);

		// Parse the response into a Map
		Map responseMap = this.parseResponseData(response);

		// Handle the response - errors first
		Map result = new HashMap();
		String resultCode = (String) responseMap.get(PayPalConstants.RESULT);

		if (resultCode == null) {
			result.put("resultCode", "10000");
			return result;
		} else if (!resultCode.equals("0")) {
			result.put("resultCode", resultCode);
			result.put("resultMessage", (String) responseMap
					.get(PayPalConstants.RESPMSG));
			return result;
		}

		// Successful transaction - write it to the database
		processSuccessfulTransaction(parameters, responseMap);

		result.put("resultCode", "0");
		return result;
	}

	protected void setPaymentVariablesToPersist(Payment payment,
			Map parameters, Map responseMap) {
		payment
				.setValueFieldLabels("ACCTTYPE, ABA, ACCTNUM, Description, RESULT, PNREF,RESPMSG");

		payment.setValue1((String) parameters.get("acctType"));
		payment.setValue2((String) parameters.get("ABARoutingNumber"));
		payment.setValue3((String) parameters.get("acctNumber"));
		payment.setValue4("Direct Payment from Bank Account");
		payment.setValue5((String) responseMap.get(PayPalConstants.RESULT));
		payment.setValue6((String) responseMap.get(PayPalConstants.PNREF));
		payment.setValue7((String) responseMap.get(PayPalConstants.RESPMSG));
		payment.setOrder(getUser().getOrder());

	}

	protected Map preparePaypalRequestAcutalPayload(Map params, Map parameters) {
		if (null == params)
			params = new HashMap();

		String firstName = "";
		String lastName = "";

		if (getUser().getOrder().getFirstName() != null)
			firstName = getUser().getOrder().getFirstName();
		if (getUser().getOrder().getLastName() != null)
			lastName += " " + getUser().getOrder().getLastName();

		params.put(PayPalConstants.FIRSTNAME, firstName + lastName);
		params.put(PayPalConstants.ABA, ((String) parameters
				.get("ABARoutingNumber")));
		params.put(PayPalConstants.ACCTTYPE, ((String) parameters
				.get("acctType")));
		params.put(PayPalConstants.ACCT, (String) parameters.get("acctNumber"));
		params.put(PayPalConstants.AUTHTYPE, "WEB");
		params.put(PayPalConstants.CHKNUM, (String) parameters.get("checkNum"));

		return params;
	}

}
