package com.softslate.commerce.businessobjects.product;

import java.util.Map;

import org.apache.lucene.document.Document;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for classes that create the Lucene index for advanced search
 * functionality.
 * 
 * When the application needs to create an instance that implements
 * <code>LuceneIndexer</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "luceneIndexerImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "luceneIndexerImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.BasicLuceneIndexer}.
 * 
 * @author Jason McWilliams, David Tobey
 * 
 */
public interface LuceneIndexer extends BusinessProcessor {

	abstract Map reindex(Map parameters) throws Exception;
	
	abstract void addStringFields(Document doc, Object object,
			StringBuffer contentBuffer) throws Exception;

	abstract void addStringFields(Document doc, Object object, StringBuffer contentBuffer, String[] fieldsToIndex)
			throws Exception;

	abstract Map addEditProduct(int productID, String indexDir) throws Exception;

	abstract Map addEditCategory(int categoryID, String indexDir) throws Exception;

	abstract Map addEditManufacturer(int manufacturerID, String indexDir) throws Exception;
}
