package com.softslate.commerce.businessobjects.shipping;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing a shipping rule range within the system.
 * <p>
 * When the application needs to create an instance that implements
 * <code>ShippingRuleRange</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "shippingRuleRangeImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "shippingRuleRangeImplementer" is
 * {@link com.softslate.commerce.businessobjects.shipping.ShippingRuleRangeBean}.
 * 
 * @author Jason McWilliams
 * 
 */
@Audited
@AuditTable("AudShippingRuleRange")
public interface ShippingRuleRange extends BusinessObject {

	int getShippingRuleRangeID();
	void setShippingRuleRangeID(int shippingRuleRangeID);
	
	int getShippingRuleID();
	void setShippingRuleID(int shippingRuleID);

	Double getFloor();
	void setFloor(Double floor);
	
	Double getCeiling();
	void setCeiling(Double ceiling);
	
	boolean getIsActive();
	void setIsActive(boolean isActive);
	
	abstract String getDiscountType();
	abstract void setDiscountType(String discountType);
	
	abstract Double getDiscount();
	abstract void setDiscount(Double discount);

	abstract String getPkgDescription();
	abstract void setPkgDescription(String pkgDescription);

	abstract Double getPkgWeight();
	abstract void setPkgWeight(Double pkgWeight);
	
	abstract Double getPkgLength();
	abstract void setPkgLength(Double pkgLength);
	
	abstract Double getPkgHeight();
	abstract void setPkgHeight(Double pkgHeight);
	
	abstract Double getPkgWidth();
	abstract void setPkgWidth(Double pkgWidth);
	
	abstract String getLimitMethods();
	abstract void setLimitMethods(String limitMethods);
	
	abstract String getLimitType();
	abstract void setLimitType(String limitType);
	
	abstract String getPkgExtra1();
	abstract void setPkgExtra1(String pkgExtra1);
	
	abstract String getPkgExtra2();
	abstract void setPkgExtra2(String pkgExtra2);
	
	abstract String getPkgExtra3();
	abstract void setPkgExtra3(String pkgExtra3);
	
	abstract String getPkgExtra4();
	abstract void setPkgExtra4(String pkgExtra4);
	
	abstract String getPkgExtra5();
	abstract void setPkgExtra5(String pkgExtra5);
	
	ShippingRule getShippingRule();
	void setShippingRule(ShippingRule shippingRule);
	
	abstract String getCountries();
	abstract void setCountries(String countries);
	
	abstract String getStates();
	abstract void setStates(String states);

	
}
