package com.softslate.commerce.businessobjects.tax;

import java.io.Serializable;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;
import com.softslate.commerce.businessobjects.order.Country;
import com.softslate.commerce.businessobjects.order.State;

/**
 * Object representing a basic tax rate within the system.
 * <code>BasicTaxRateBean</code> is the default implementation of the
 * <code>BasicTaxRate</code> interface for the application.
 * <p>
 * The database stores basic tax rates in the <code>sscBasicTaxRate</code>
 * table, and this interface can be used to represent a single row of that
 * table. The Hibernate configuration file <code>BasicTaxRateBean.hbm.xml</code>
 * maps that table's columns to the properties in this object.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudBasicTaxRate")
public class BasicTaxRateBean extends BaseBusinessObject implements
		BasicTaxRate, Serializable {

	private static final long serialVersionUID = -7705321675066023730L;

	private int basicTaxRateID = 0;

	private Integer countryID = null;

	private Integer stateID = null;

	private String city = null;

	private String postalCode = null;

	private Double rate = null;

	private boolean isActive = true;

	private Country country = null;

	private State state = null;

	public Country getCountry() {
		return country;
	}

	public void setCountry(Country country) {
		this.country = country;
	}

	public State getState() {
		return state;
	}

	public void setState(State state) {
		this.state = state;
	}

	public int getBasicTaxRateID() {
		return basicTaxRateID;
	}

	public void setBasicTaxRateID(int basicTaxRateID) {
		this.basicTaxRateID = basicTaxRateID;
	}

	public String getCity() {
		return city;
	}

	public void setCity(String city) {
		this.city = city;
	}

	public Integer getCountryID() {
		return countryID;
	}

	public void setCountryID(Integer countryID) {
		this.countryID = countryID;
	}

	public boolean getIsActive() {
		return isActive;
	}

	public void setIsActive(boolean isActive) {
		this.isActive = isActive;
	}

	public String getPostalCode() {
		return postalCode;
	}

	public void setPostalCode(String postalCode) {
		this.postalCode = postalCode;
	}

	public Double getRate() {
		return rate;
	}

	public void setRate(Double rate) {
		this.rate = rate;
	}

	public Integer getStateID() {
		return stateID;
	}

	public void setStateID(Integer stateID) {
		this.stateID = stateID;
	}
}
