package com.softslate.commerce.customer.core;

import java.io.BufferedReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.JsonPrimitive;

/**
 * @author David Tobey
 */
public class APIRequestWrapper extends HttpServletRequestWrapper {

	static Log log = LogFactory.getLog(APIRequestWrapper.class);

	private HttpServletRequest originalRequest;

	private Map<String, String[]> parameterMap;

	public APIRequestWrapper(HttpServletRequest request) {
		super(request);
		this.originalRequest = request;
		String contentType = request.getHeader("Content-Type");
		if (contentType == null || !contentType.equals("application/x-www-form-urlencoded")) {
			convertJsonPayloadToParameters(request);
		}
	}

	private void convertJsonPayloadToParameters(HttpServletRequest request) {
		StringBuffer sb = new StringBuffer();
		String line = null;
		try {
			BufferedReader reader = request.getReader();
			while ((line = reader.readLine()) != null) {
				sb.append(line);
			}
			if (sb.length() > 0) {
				JsonElement jelement = new JsonParser().parse(sb.toString());
				if (jelement != null) {
					JsonObject jobject = jelement.getAsJsonObject();
					log.debug("jobject: " + jobject);
					Set<Map.Entry<String, JsonElement>> members = jobject.entrySet();
					for (Map.Entry<String, JsonElement> member : members) {
						JsonElement element = member.getValue();
						if (element.isJsonPrimitive()) {
							JsonPrimitive primitive = element.getAsJsonPrimitive();
							String value = null;
							if (primitive.isBoolean()) {
								value = primitive.toString();
							} else if (primitive.isNumber()) {
								value = primitive.toString();
							} else if (primitive.isString()) {
								value = primitive.getAsString();
							}
							if (value != null) {
								log.debug("name: " + member.getKey() + ", value: " + value);
								addParameter(member.getKey(), value);
							}
						}
					}
				}
			}
		} catch (Exception e) {
			log.info("Skipping conversion from json input: " + e.toString());
		}
	}

	@Override
	public String getMethod() {
		if (super.getMethod().equals("PUT") || super.getMethod().equals("PATCH")) {
			return "POST";
		} else if (super.getMethod().equals("DELETE")) {
			return "GET";
		} else {
			return super.getMethod();
		}
	}

	@Override
	public String getParameter(String name) {
		if (parameterMap == null) {
			return originalRequest.getParameter(name);
		}
		String[] strings = parameterMap.get(name);
		if (strings != null) {
			return strings[0];
		}
		return null;
	}

	@Override
	public Map<String, String[]> getParameterMap() {
		if (parameterMap == null) {
			return originalRequest.getParameterMap();
		}
		return Collections.unmodifiableMap(parameterMap);
	}

	@Override
	public Enumeration<String> getParameterNames() {
		if (parameterMap == null) {
			return originalRequest.getParameterNames();
		}
		return Collections.enumeration(parameterMap.keySet());
	}

	@Override
	public String[] getParameterValues(String name) {
		if (parameterMap == null) {
			return originalRequest.getParameterValues(name);
		}
		return parameterMap.get(name);
	}

	public void addParameter(String name, String value) {
		if (parameterMap == null) {
			parameterMap = new HashMap<String, String[]>();
			parameterMap.putAll(originalRequest.getParameterMap());
		}
		String[] values = parameterMap.get(name);
		if (values == null) {
			values = new String[0];
		}
		List<String> list = new ArrayList<String>(values.length + 1);
		list.addAll(Arrays.asList(values));
		list.add(value);
		parameterMap.put(name, list.toArray(new String[0]));
	}

}
