package com.softslate.commerce.customer.payment;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.customer.core.ActionUtils;

/**
 * Struts form class for submissions of payment information for the First Data
 * Global Gateway during checkout. This form corresponds to the
 * {@link com.softslate.commerce.businessobjects.payment.FirstDataProcessor}
 * payment processor.
 * <p>
 * The database settings "activePaymentProcessors", "activePaymentForms", and
 * "activePaymentFormTemplate" represent the various payment methods available
 * to users. During checkout, each of the "activePaymentForms" will be
 * instantiated and prepared for display on the checkout payment screen. When
 * the user submits payment information the <code>validate</code> method of the
 * corresponding payment form is invoked.
 * <p>
 * The <code>validate</code> method of this class ensures that the incoming
 * credit card information is valid and complete. It then parses the database
 * settings for various PayflowLink fields and populates the corresponding form
 * properties.
 * 
 * @author David Tobey
 */
public class FirstDataForm extends BasicPaymentForm {

	private static final long serialVersionUID = 2846001494308159452L;

	static Log log = LogFactory.getLog(FirstDataForm.class);

	private String certPath = null;

	private String ipAddress = null;

	private String comments = null;

	public String getCertPath() {
		return certPath;
	}

	public void setCertPath(String certPath) {
		this.certPath = certPath;
	}

	public String getIpAddress() {
		return ipAddress;
	}

	public void setIpAddress(String ipAddress) {
		this.ipAddress = ipAddress;
	}

	public String getComments() {
		return comments;
	}

	public void setComments(String comments) {
		this.comments = comments;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);
		super.validate(mapping, request);
		return firstDataValidate();
	}

	public ActionErrors firstDataValidate() {
		ActionUtils actionUtils = (ActionUtils) getBusinessObjectFactory()
				.createUtilsObject("actionUtilsImplementer");

		// Set the certificate path
		String firstDataClientCertificatePath = (String) getSettings()
				.getValue("firstDataClientCertificatePath");
		if (firstDataClientCertificatePath.startsWith("/WEB-INF")) {
			setCertPath(getServlet().getServletContext().getRealPath(
					firstDataClientCertificatePath));
		} else {
			setCertPath(firstDataClientCertificatePath);
		}

		// Set the IP address
		setIpAddress(getRequest().getRemoteAddr());

		// Populate comments, parsing them for variables.
		try {
			setComments(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("firstDataComments")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		if (log.isDebugEnabled()) {
			log.debug("comments: " + getComments());
			log.debug("ipAddress: " + getIpAddress());
			log.debug("Cert path: " + getCertPath());
		}
		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());

		return getErrors();
	}

}
