package com.softslate.commerce.daos.product;

import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.SKUDiscount;
import com.softslate.commerce.daos.core.BaseAdminGatewayDAOHibernate;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO that runs miscellaneous queries related to discounts.
 * 
 * @author David Tobey
 */
public class DiscountGatewayDAOHibernate extends BaseAdminGatewayDAOHibernate
		implements DiscountGatewayDAO {

	static Log log = LogFactory.getLog(DiscountGatewayDAOHibernate.class);

	public Collection loadStoreDiscounts() throws Exception {
		Collection results = null;
		Session session = getDaoFactory().startSession();
		session.clear();
		try {
			Query q = getNamedQuery(session, "discounts.selectStoreDiscounts");
			results = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			log.error("Hibernate error: " + he.toString(), he);
		}
		return results;
	}

	public Map loadDiscountsAndCount(Map parameters) throws Exception {
		return loadItemsAndCount(parameters);
	}

	public Map editDeleteDiscounts(Map parameters, BusinessObjectFactory bof)
			throws Exception {
		Map result = editDeleteItems(parameters, bof);
		evictCache();
		return result;
	}

	public Map editDeleteAssignments(Map parameters, Object item,
			BusinessObjectFactory bof, int index) throws Exception {
		Map result = null;
		String idField = (String) parameters.get("idField");
		Integer[] idIntegers = (Integer[]) parameters.get("idIntegers");
		String parentIDField = (String) parameters.get("parentIDField");
		String parentID = (String) parameters.get("parentID");
		String manyToManyKey = (String) parameters.get("manyToManyKey");
		String manyToManyClass = (String) parameters.get("manyToManyClass");
		String entityProperty = (String) parameters.get("entityProperty");
		String[] assigned = (String[]) parameters.get("assigned");
		String[] skuIDs = (String[]) parameters.get("skuIDs");

		// Just do special processing if this is the list of discounts under a
		// certain product.
		if (parentID == null || !parentIDField.equals("productID")) {
			return super.editDeleteAssignments(parameters, item, bof, index);
		}

		Session session = getDaoFactory().getSession();

		// Add and delete assignments
		if (assigned != null && assigned.length > index) {

			// Loop through each of the skuID's present in the skuIDs parameter,
			// which is a pipe-separated list of skuIDs under the product
			// associated with the discount.
			String skuIDsPipeList = skuIDs[index];
			String[] skuIDsArray = skuIDsPipeList.split("\\|");
			for (int i = 0; i < skuIDsArray.length; i++) {
				String skuID = skuIDsArray[i];
				skuID = skuID.trim();
				if (skuID.length() > 0) {
					if (log.isDebugEnabled())
						log.debug("index:" + index + " skuID: " + skuID
								+ " discountID: " + idIntegers[index]);

					Class m2m = Class.forName(manyToManyClass);
					Criteria m2mCrit = session.createCriteria(m2m);
					m2mCrit.add(Restrictions.eq(idField, idIntegers[index]));
					m2mCrit.add(Restrictions.eq("skuID", new Integer(Integer
							.parseInt(skuID))));
					Object o = m2mCrit.uniqueResult();
					boolean deleteAssignment = o != null
							&& (assigned[index].equals("0") || assigned[index]
									.equals("false"));
					boolean insertAssignmeent = o == null
							&& (assigned[index].equals("1") || assigned[index]
									.equals("true"));
					boolean updateAssignment = o != null && !deleteAssignment;
					if (log.isDebugEnabled())
						log.debug("Delete/Insert/Update assignment: "
								+ deleteAssignment + "/" + insertAssignmeent
								+ "/" + updateAssignment);

					if (deleteAssignment) {

						// Delete the assignment
						session.delete(o);
					} else if (insertAssignmeent || updateAssignment) {

						// Insert new or update existing
						if (insertAssignmeent) {
							o = bof.createObject(manyToManyKey);
						}
						Object p = bof.createObject("skuImplementer");
						BeanUtils.copyProperty(p, "skuID", skuID);
						BeanUtils.copyProperty(o, "sku", p);
						BeanUtils.copyProperty(o, entityProperty, item);
						session.saveOrUpdate(o);
					}
				}
			}
		}
		return result;
	}

	public void insertSKUDiscount(SKUDiscount sd) throws DataAccessException {
		insertSKUDiscount(sd, false);
	}

	public void insertSKUDiscount(SKUDiscount sd, boolean commit)
			throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			session.save(sd);
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			evictCache();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public Collection loadAllCoupons() throws DataAccessException {
		Collection coupons = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "discount.selectAllCoupons");
			coupons = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return coupons;
	}
}
