package com.softslate.commerce.daos.product;

import java.util.LinkedHashSet;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.product.Manufacturer;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscManufacturer</code> table of the
 * database.
 * 
 * @author David Tobey
 * 
 */
public class ManufacturerDAOHibernate extends BaseDAO implements
		ManufacturerDAO {

	static Log log = LogFactory.getLog(ManufacturerDAOHibernate.class);

	private Manufacturer manufacturer = null;

	public Manufacturer getManufacturer() {
		return (this.manufacturer);
	}

	public void setManufacturer(Manufacturer manufacturer) {
		this.manufacturer = manufacturer;
	}

	public void loadManufacturerFromID() throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading manufacturer: " + getManufacturer().getCode());
		Session session = getDaoFactory().startSession();
		try {
			Manufacturer manufacturer = (Manufacturer) session.get(
					Manufacturer.class, new Integer(getManufacturer()
							.getManufacturerID()));
			if (manufacturer != null) {
				PropertyUtils.copyProperties(getManufacturer(), manufacturer);
			}
			setManufacturer(manufacturer);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void loadManufacturer() throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "manufacturer.selectFromCode");
			q.setString("code", getManufacturer().getCode());
			q.setCacheable(true);
			Manufacturer loadedManufacturer = (Manufacturer) q.uniqueResult();
			setManufacturer(loadedManufacturer);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void insertManufacturer() throws DataAccessException {
		insertManufacturer(false);
	}

	public void insertManufacturer(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			session.save(getManufacturer());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			evictCache();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void updateManufacturer() throws DataAccessException {
		updateManufacturer(false);
	}

	public void updateManufacturer(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			session.update(getManufacturer());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void deleteManufacturer() throws DataAccessException {
		deleteManufacturer(false);
	}

	public void deleteManufacturer(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			setManufacturer((Manufacturer) session.load(Manufacturer.class,
					new Integer(getManufacturer().getManufacturerID())));
			session.delete(getManufacturer());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			evictCache();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void loadProducts() throws DataAccessException {
		if (log.isInfoEnabled())
			log.info("Loading products: " + getManufacturer().getCode());
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "manufacturer.selectProductList");
			q.setInteger("manufacturerID", getManufacturer()
					.getManufacturerID());
			q.setCacheable(true);
			getManufacturer().setProducts(new LinkedHashSet(q.list()));
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public Object loadObject(Object object) throws Exception {
		if (null != object && object instanceof Manufacturer) {
			this.setManufacturer((Manufacturer) object);
			if (0 != manufacturer.getManufacturerID()) {
				this.loadManufacturerFromID();
			} else if (null != manufacturer.getCode()) {
				this.loadManufacturer();
			}
			return this.getManufacturer();
		}
		return null;
	}
}
